#!/usr/bin/env python
# -*- coding: utf-8 -*-
"""enums - Enumerates for apt daemon dbus messages"""
# Copyright (C) 2008-2009 Sebastian Heinlein <devel@glatzor.de>
#
# This program is free software; you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation; either version 2 of the License, or
# any later version.
#
# This program is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
#
# You should have received a copy of the GNU General Public License along
# with this program; if not, write to the Free Software Foundation, Inc.,
# 51 Franklin Street, Fifth Floor, Boston, MA 02110-1301 USA.

__author__  = "Sebastian Heinlein <devel@glatzor.de>"

import gettext
def _(msg):
   return gettext.dgettext("aptdaemon", msg)

# Finish states
(EXIT_SUCCESS,
 EXIT_CANCELLED,
 EXIT_FAILED) = range(3)

# Error codes
(ERROR_PACKAGE_DOWNLOAD_FAILED,
 ERROR_REPO_DOWNLOAD_FAILED,
 ERROR_DEP_RESOLUTION_FAILED,
 ERROR_KEY_NOT_INSTALLED,
 ERROR_KEY_NOT_REMOVED,
 ERROR_NO_LOCK,
 ERROR_NO_CACHE,
 ERROR_NO_PACKAGE,
 ERROR_PACKAGE_UPTODATE,
 ERROR_PACKAGE_ALREADY_INSTALLED,
 ERROR_PACKAGE_NOT_INSTALLED,
 ERROR_NOT_REMOVE_ESSENTIAL_PACKAGE,
 ERROR_DAEMON_DIED,
 ERROR_PACKAGE_MANAGER_FAILED,
 ERROR_CACHE_BROKEN,
 ERROR_PACKAGE_UNAUTHENTICATED,
 ERROR_UNKNOWN) = range(17)

# Message enums
(MSG_SYSTEM_ALREADY_UPTODATE,
 MSG_SYSTEM_NOT_UPTODATE) = range(2)

# Status codes
(STATUS_SETTING_UP,
 STATUS_WAITING,
 STATUS_WAITING_MEDIUM,
 STATUS_WAITING_CONFIG_FILE_PROMPT,
 STATUS_WAITING_LOCK,
 STATUS_RUNNING,
 STATUS_LOADING_CACHE,
 STATUS_DOWNLOADING,
 STATUS_COMMITTING,
 STATUS_CLEANING_UP,
 STATUS_RESOLVING_DEP,
 STATUS_FINISHED,
 STATUS_CANCELLING) = range(13)

(ROLE_UNSET,
 ROLE_INSTALL_PACKAGES,
 ROLE_INSTALL_FILE,
 ROLE_UPGRADE_PACKAGES,
 ROLE_UPGRADE_SYSTEM,
 ROLE_UPDATE_CACHE,
 ROLE_REMOVE_PACKAGES,
 ROLE_COMMIT_PACKAGES,
 ROLE_ADD_VENDOR_KEY_FILE,
 ROLE_REMOVE_VENDOR_KEY) = range(10)

ICONS_STATUS = {
    STATUS_CANCELLING:'aptdaemon-cleanup',
    STATUS_CLEANING_UP:'aptdaemon-cleanup',
    STATUS_RESOLVING_DEP:'aptdaemon-resolve',
    STATUS_COMMITTING:'aptdaemon-setup',
    STATUS_DOWNLOADING:'aptdaemon-download',
    STATUS_FINISHED:'aptdaemon-cleanup',
    STATUS_LOADING_CACHE:'aptdaemon-update-cache',
    STATUS_RUNNING:'aptdaemon-setup',
    STATUS_SETTING_UP:'aptdaemon-setup',
    STATUS_WAITING:'aptdaemon-wait',
    STATUS_WAITING_LOCK:'aptdaemon-wait',
    STATUS_WAITING_MEDIUM:'aptdaemon-wait',
    STATUS_WAITING_CONFIG_FILE_PROMPT:'aptdaemon-wait',
    }

def get_status_icon_name_from_enum(enum):
   if ICONS_STATUS.has_key(enum):
       return ICONS_STATUS[enum]
   else:
       return None

ICONS_ROLE = {
    ROLE_INSTALL_FILE:'aptdaemon-add',
    ROLE_INSTALL_PACKAGES:'aptdaemon-add',
    ROLE_UPDATE_CACHE:'aptdaemon-update-cache',
    ROLE_REMOVE_PACKAGES:'aptdaemon-delete',
    ROLE_UPGRADE_PACKAGES:'aptdaemon-upgrade',
    ROLE_UPGRADE_SYSTEM:'system-software-update',
    }

def get_role_icon_name_from_enum(enum):
   if ICONS_ROLE.has_key(enum):
       return ICONS_ROLE[enum]
   else:
       return None

ANIMATIONS_STATUS = {
    STATUS_CANCELLING:'aptdaemon-action-cleaning-up',
    STATUS_CLEANING_UP:'aptdaemon-action-cleaning-up',
    STATUS_RESOLVING_DEP:'aptdaemon-action-resolving',
    STATUS_DOWNLOADING:'aptdaemon-action-downloading',
    STATUS_LOADING_CACHE:'aptdaemon-action-updating-cache',
    STATUS_WAITING:'aptdaemon-action-waiting',
    STATUS_WAITING_LOCK:'aptdaemon-action-waiting',
    STATUS_WAITING_MEDIUM:'aptdaemon-action-waiting',
    STATUS_WAITING_CONFIG_FILE_PROMPT:'aptdaemon-action-waiting',
    }


def get_status_animation_name_from_enum(enum):
   if ANIMATIONS_STATUS.has_key(enum):
       return ANIMATIONS_STATUS[enum]
   else:
       return None

PAST_ROLE = {
    ROLE_INSTALL_FILE : _("Installed file"),
    ROLE_INSTALL_PACKAGES : _("Installed packages"),
    ROLE_ADD_VENDOR_KEY_FILE: _("Added key from file"),
    ROLE_UPDATE_CACHE : _("Updated cache"),
    ROLE_REMOVE_VENDOR_KEY : _("Removed trusted key"),
    ROLE_REMOVE_PACKAGES : _("Removed packages"),
    ROLE_UPGRADE_PACKAGES : _("Updated packages"),
    ROLE_UPGRADE_SYSTEM : _("Upgraded system"),
    ROLE_COMMIT_PACKAGES : _("Applied changes"),
    ROLE_UNSET : ""
    }

def get_role_localised_past_from_enum(enum):
   if PAST_ROLE.has_key(enum):
       return PAST_ROLE[enum]
   else:
       return None

STRING_EXIT = {
    EXIT_SUCCESS : _("Successful"),
    EXIT_CANCELLED : _("Canceled"),
    EXIT_FAILED : _("Failed")
    }

def get_exit_string_from_enum(enum):
   if STRING_EXIT.has_key(enum):
       return STRING_EXIT[enum]
   else:
       return None

PRESENT_ROLE = {
    ROLE_INSTALL_FILE : _("Installing file"),
    ROLE_INSTALL_PACKAGES : _("Installing packages"),
    ROLE_ADD_VENDOR_KEY_FILE : _("Adding key from file"),
    ROLE_UPDATE_CACHE : _("Updating cache"),
    ROLE_REMOVE_VENDOR_KEY : _("Removing trusted key"),
    ROLE_REMOVE_PACKAGES : _("Removing packages"),
    ROLE_UPGRADE_PACKAGES : _("Updating packages"),
    ROLE_UPGRADE_SYSTEM : _("Upgrading system"),
    ROLE_COMMIT_PACKAGES : _("Applying changes"),
    ROLE_UNSET : ""
    }

def get_role_localised_present_from_enum(enum):
   if PRESENT_ROLE.has_key(enum):
       return PRESENT_ROLE[enum]
   else:
       return None

DESCS_ERROR = {
    ERROR_PACKAGE_DOWNLOAD_FAILED : _("Check your Internet connection."),
    ERROR_REPO_DOWNLOAD_FAILED : _("Check your Internet connection."),
    ERROR_CACHE_BROKEN : _("Check if you are using third party "
                           "repositories. If so disable them, since "
                           "they are a common source of problems.\n"
                           "Furthermore run the following command in a "
                           "Terminal: apt-get install -f"),
    ERROR_KEY_NOT_INSTALLED : _("The selected file may not be a GPG key file "
                                "or it might be corrupt."),
    ERROR_KEY_NOT_REMOVED : _("The selected key couldn't be removed "
                               "Check if you provided a valid fingerprint."),
    ERROR_NO_LOCK : _("Check if you are currently running another "
                      "software management tool, e.g. Synaptic or aptitude. "
                      "Only one tool is allowed to make changes at the "
                      "same time."),
    ERROR_NO_CACHE : _("This is a serious problem. Try again later. If this "
                       "problem appears again, please report an error to the "
                       "developers."),
    ERROR_NO_PACKAGE : _("Check the spelling of the package name and if you "
                         "have got enabled the corresponding repository."),
    ERROR_PACKAGE_UPTODATE : _("There isn't any need for an update."),
    ERROR_PACKAGE_ALREADY_INSTALLED : _("There isn't any need for an "
                                        "installation"),
    ERROR_PACKAGE_NOT_INSTALLED : _("There isn't any need for a removal."),
    ERROR_NOT_REMOVE_ESSENTIAL_PACKAGE : _("You requested to remove a package "
                                           "which is an essential part of your "
                                           "system."),
    ERROR_DAEMON_DIED : _("The connection to the daemon was lost. Most likely "
                          "the background daemon crashed."),
    ERROR_PACKAGE_MANAGER_FAILED: _("The installation or removal of a software "
                                    "package failed."),
    ERROR_UNKNOWN : _("This should not happen."),
    ERROR_DEP_RESOLUTION_FAILED: _("This error could be caused by required "
                                   "additional software packages which are "
                                   "missing or not installable. Futhermore "
                                   "there could be a conflict between software "
                                   "packages which are not allowed to be "
                                   "installed at the same time."),
    ERROR_PACKAGE_UNAUTHENTICATED: _("The action would require the "
                                     "installation of packages from not "
                                     "authenticated sources.")
    }

def get_error_description_from_enum(enum):
   if DESCS_ERROR.has_key(enum):
       return DESCS_ERROR[enum]
   else:
       return None

STRINGS_ERROR = {
    ERROR_PACKAGE_DOWNLOAD_FAILED : _("Failed to download package files"),
    ERROR_REPO_DOWNLOAD_FAILED : _("Failed to download repository information"),
    ERROR_DEP_RESOLUTION_FAILED : _("Package dependencies cannot be resolved"),
    ERROR_CACHE_BROKEN : _("The package system is broken"),
    ERROR_KEY_NOT_INSTALLED : _("Key was not installed"),
    ERROR_KEY_NOT_INSTALLED : _("Key was not removed"),
    ERROR_NO_LOCK : _("Failed to lock the package manager"),
    ERROR_NO_CACHE : _("Failed to load the package list"),
    ERROR_NO_PACKAGE : _("Package does not exist"),
    ERROR_PACKAGE_UPTODATE : _("Package is already up-to-date"),
    ERROR_PACKAGE_ALREADY_INSTALLED : _("Package is already installed"),
    ERROR_PACKAGE_NOT_INSTALLED : _("Package isn't installed"),
    ERROR_NOT_REMOVE_ESSENTIAL_PACKAGE : _("Failed to remove essential "
                                           "system package"),
    ERROR_DAEMON_DIED : _("Task cannot be monitored or controlled"),
    ERROR_PACKAGE_MANAGER_FAILED: _("Package operation failed"),
    ERROR_PACKAGE_UNAUTHENTICATED: _("Requires installation of untrusted "
                                     "packages"),
    ERROR_UNKNOWN : _("An unknown error occurred")
    }

def get_error_string_from_enum(enum):
   if STRINGS_ERROR.has_key(enum):
       return STRINGS_ERROR[enum]
   else:
       return None

STRINGS_MESSAGE = {
    MSG_SYSTEM_ALREADY_UPTODATE : _("System is already up-to-date"),
    MSG_SYSTEM_NOT_UPTODATE : _("System is not up-to-date")
    }

def get_msg_string_from_enum(enum):
   if STRINGS_MESSAGE.has_key(enum):
       return STRINGS_MESSAGE[enum]
   else:
       return None

DESCS_MESSAGE = {
    MSG_SYSTEM_ALREADY_UPTODATE : _("There isn't any need for a system update"),
    MSG_SYSTEM_NOT_UPTODATE : _("It was not possible to install the latest "
                                "for all packages.")
    }

def get_msg_description_from_enum(enum):
   if DESCS_MESSAGE.has_key(enum):
       return DESCS_MESSAGE[enum]
   else:
       return None

STRINGS_STATUS = {
    STATUS_SETTING_UP : _("Waiting for service to start"),
    STATUS_WAITING : _("Waiting for other tasks"),
    STATUS_WAITING_MEDIUM : _("Waiting for required medium"),
    STATUS_WAITING_LOCK : _("Waiting for other software managers to quit"),
    STATUS_WAITING_CONFIG_FILE_PROMPT : _("Waiting for configuration file prompt"),
    STATUS_RUNNING : _("Running task"),
    STATUS_DOWNLOADING : _("Downloading"),
    STATUS_CLEANING_UP : _("Cleaning up"),
    STATUS_RESOLVING_DEP : _("Resolving dependencies"),
    STATUS_COMMITTING : _("Applying changes"),
    STATUS_FINISHED : _("Finished"),
    STATUS_CANCELLING : _("Cancelling"),
    STATUS_LOADING_CACHE : _("Loading software list"),
    }

def get_status_string_from_enum(enum):
   if STRINGS_STATUS.has_key(enum):
       return STRINGS_STATUS[enum]
   else:
       return None

# vim:ts=4:sw=4:et
